﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ZedGraph;

namespace EPR_deRaedt
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
            InitialiseCalulation();
        }
        private void InitialiseCalulation()
        {
            foreach (Epr.Calculation x in Enum.GetValues(typeof(Epr.Calculation)))
            {
                cbxCalculation.Items.Add(x);
            }
            cbxCalculation.SelectedItem = Epr.Calc_Epr.Interpretation;
        }

        private void btnCalc_Click(object sender, EventArgs e)
        {
            this.Cursor = Cursors.WaitCursor;
            btnCalc.Enabled = false;
            this.Refresh();
            Epr.Calc_Epr CalcEpr = new Epr.Calc_Epr();
            List<Epr.Result> results= CalcEpr.Go();
            DrawWithZedGraph(results);
            this.Cursor = Cursors.Default;
            btnCalc.Enabled = true;
        }
  
        private void cbxCalculation_SelectedIndexChanged(object sender, EventArgs e)
        {
            Epr.Calc_Epr.Interpretation = (Epr.Calculation) cbxCalculation.SelectedItem;
        }
        //use ZedGraph
        private void DrawWithZedGraph(List<Epr.Result> results)
        {
            GraphPane myPane = this.zedGraphControl1.GraphPane;
            // Set the titles and axis labels
            myPane.Title.Text = "EPRB";
            if (Epr.Calc_Epr.Sphi > 0)
            {
                myPane.Title.Text += " psi= " + Epr.Calc_Epr.Sphi.ToString("F"); 
            }
            myPane.XAxis.Title.Text = "Angle";
            myPane.YAxis.Title.Text = "Coincidence";
            //myPane.Legend.
            PointPairList list = new PointPairList();
           if (Epr.Calc_Epr.Interpretation == Epr.Calculation.DeReat)
            {
                foreach (Epr.Result result in results)
                {
                    list.Add( Math.Abs(result.Angle), result.Eab_W0);
                }
            }
            else
            {
                foreach (Epr.Result result in results)
                {
                    list.Add( Math.Abs(result.Angle), result.Eab_Wt0);
                }
            }
           list.Sort();
				//myPane.

			// Generate a blue curve with circle symbols, and "My Curve 2" in the legend
           LineItem myCurve = myPane.AddCurve(Epr.Calc_Epr.Interpretation.ToString(), list, GetRangeColor(),
									SymbolType.HDash );
            myCurve.Line.Color = Color.Transparent;
            //
            if (!cos2PhiAdded)
            {
                PointPairList list2 = new PointPairList();
                for (double i = 0; i < 181; i = i + 5)
                {
                    double phi = Epr.h.GetAngleAsPi(i);
                    double y = -Math.Cos(2 * phi);
                    list2.Add(i, y);
                }
                LineItem myCurve2 = myPane.AddCurve("-cos(2phi)", list2, Color.Black,
                                         SymbolType.None);
                cos2PhiAdded = true;
            }
             this.zedGraphControl1.AxisChange();
             this.Refresh();
        }
        private bool cos2PhiAdded = false;
        private Color GetRangeColor()
        {
            switch (Epr.Calc_Epr.Interpretation)
            {
                case Epr.Calculation.DeReat:
                    return Color.Blue;
                    break;
                case Epr.Calculation.Malus:
                    return Color.Green;
                    break;
                case Epr.Calculation.Test:
                    return Color.Yellow;
                    break;

            }
            return Color.Black;
        }
    }

}
