﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Epr
{
    public class Calc_Epr
    {
        int NumberOfAnglesForStations = 9; // number of angles at station 1 and 2 + 1
        public static double backGroundPsi = 0;
        // model parameters
        int N = 100000; // Number of events generated by the source for each pair of angles
        double tau = 0.0025; // time-tag resolution in units of T0 = 1, see text
        int k = 1; // time window W = k * tau
        int d = 2; // parameter for the time-tag model
        public static Calculation Interpretation = Calculation.DeReat;
        //
        public static double Sphi = 0;
        // initialization of the random number generator
        private int[, , ,] getTimingArray()
        {
            return new int[2, 2, NumberOfAnglesForStations, NumberOfAnglesForStations];
        }


        public List<Result> Go()
        {
            List<Result> results = new List<Result>();
            //
            // Program to simulate an Einstein-Podolsky-Rosen-Bohm experiment
            //
            //int Mdim = M + 1;
            int[, , ,] Notiming = getTimingArray(); //integer :: Notiming(0:1,0:1,0:M,0:M) // counts without using time-tag data
            int[, , ,] Ntiming = getTimingArray();//integer :: Ntiming(0:1,0:1,0:M,0:M) // counts using time-tag data

 

            PolarizationFilter PolarizationFilter1 = new PolarizationFilter(NumberOfAnglesForStations, d, tau);
            PolarizationFilter PolarizationFilter2 = new PolarizationFilter(NumberOfAnglesForStations, d, tau);
            backGroundPsi = h.GetRandomTwoPiAngle();
            
            int ChoseFilterAnglesCounter = 0;
            // generate events
            for (int i = 0; i < N * NumberOfAnglesForStations * NumberOfAnglesForStations; i++)
            {

                //Initiate entangled particles
                Particle Particle1 = new Particle();
                Particle Particle2 = new Particle();

                //polarization relation
                Particle2.Polarization = Particle1.Polarization + h.PiOver2; // polarization of particle 2
                 int AngleIndexStation1 = 0;
                int AngleIndexStation2 = 0;
                if (true ||ChoseFilterAnglesCounter == 0)
                {
                    //Set the filters
                    AngleIndexStation1 = PolarizationFilter1.ChooseRandomAngleIndex(); // pick an angle at station 1
                    AngleIndexStation2 = PolarizationFilter2.ChooseRandomAngleIndex(); // pick an angle at station 2
                }
                ChoseFilterAnglesCounter++;
                if (ChoseFilterAnglesCounter > 100)
                {
                    ChoseFilterAnglesCounter = 0;
                }
                // station 1
                PolarizationFilter1.ParticleHit(Particle1);

                //station 2
                PolarizationFilter2.ParticleHit(Particle2);



                 
                // count
                Notiming[Particle1.Passed, Particle2.Passed, AngleIndexStation1, AngleIndexStation2] += 1; // Model without timing (W > T0)
                if (Math.Abs(Particle1.DelayTime - Particle2.DelayTime) < k) // model with timing
                {
                    Ntiming[Particle1.Passed, Particle2.Passed, AngleIndexStation1, AngleIndexStation2] += 1;
                }
            }
            //end do i=1,N*M*M // generate events
            //
            // Print
            Console.WriteLine("a-b   -cos(2*(a-b)) E(a,b,W->0) E(a,b,W>T0) E1(a,W->0) E2(b,W->0) f(a,b)  E1(a,W>T0) E2(b,W>T0):");

            bool AlsoCountTheNoHits = true;
            for (int i2 = 0; i2 < NumberOfAnglesForStations; i2++)
            {
                for (int i1 = 0; i1 < NumberOfAnglesForStations; i1++)
                {
                    double angle = PolarizationFilter1.Angles[i1] - PolarizationFilter2.Angles[i2];
                    // expectations without using time-tag information (Bell-type model)          
                    // denominator of (2)
                    double r0 = Notiming[1, 1, i1, i2] + Notiming[0, 1, i1, i2] + Notiming[1, 0, i1, i2];
                    // numerator of (2)
                    double r1 = Notiming[1, 1, i1, i2] - Notiming[0, 1, i1, i2] - Notiming[1, 0, i1, i2];
                    double r2 = Notiming[0, 1, i1, i2]; // single particle counts
                    double r3 = Notiming[1, 0, i1, i2]; // single particle counts

                    //total sample
                    double TotalSampleCount = Notiming[0, 0, i1, i2]*1;
                    if (AlsoCountTheNoHits)
                    {
                        r0 += TotalSampleCount;
                        r1 += TotalSampleCount;
                        r2 += TotalSampleCount;
                        r3 += TotalSampleCount;
                    }
                    if (r0 > 0)
                    {
                        r1 = r1 / r0;
                        r2 = r2 / r0;
                        r3 = r3 / r0;
                    }
                    // expectations using time-tag information
                    // denominator of (2)
                    double t0 =  Ntiming[1, 1, i1, i2] + Ntiming[0, 1, i1, i2] + Ntiming[1, 0, i1, i2];
                    // numerator of [2]
                    double t1 =  Ntiming[1, 1, i1, i2] - Ntiming[0, 1, i1, i2] - Ntiming[1, 0, i1, i2];
                    double t2 =  Ntiming[0, 1, i1, i2]; // single particle counts
                    double t3 =  Ntiming[1, 0, i1, i2]; // single particle counts
                    if (AlsoCountTheNoHits)
                    {
                        t0 += Ntiming[0, 0, i1, i2];
                        t1 += Ntiming[0, 0, i1, i2];
                        t2 += Ntiming[0, 0, i1, i2];
                        t3 += Ntiming[0, 0, i1, i2];
                    }
                    if (t0 > 0)
                    {
                        t1 = t1 / t0;
                        t2 = t2 / t0;
                        t3 = t3 / t0;
                    }
                    Result result = new Result();
                    double tAngle = ((double)180) * (angle / h.Pi);
                    if (tAngle < 0)
                    {
                        tAngle = 180 + tAngle;
                    }
                    result.Angle = tAngle;
                    result.Cos = -Math.Cos(2 * angle);
                    result.Eab_W0 = t1;
                    result.Eab_Wt0 = r1;
                    result.E1a_W0 = t2;
                    result.E2b_W0 = t3;
                    result.Fab = (double)t0 / (double)N;
                    result.E1a_Wt0 = r2;
                    result.E2b_Wt0 = r3;

                    results.Add(result);
                    //Console.WriteLine("a-b              -cos(2*(a-b))         E(a,b,W->0) E(a,b,W>T0) E1(a,W->0) E2(b,W->0) f(a,b)                  E1(a,W>T0) E2(b,W>T0):");
                    //                  180 * angle / pi, -Math.Cos(2 * angle), t1,         r1,         t2,        t3,        (double)t0 / (double)N, r2,        r3);
                    Console.WriteLine("{0,0:f}            {1,0:f}               {2,0:f}     {3,0:f}     {4,0:f}    {5,0:f}    {6,0:f}                 {7,0:f}    {8,0:f}", 180 * angle / h.Pi, -Math.Cos(2 * angle), t1, r1, t2, t3, (double)t0 / (double)N, r2, r3);
                }
            }
            return results;
        }
        private int FilterCheck(double Angle1, double Angle2, out double c)
        {
            c = h.Cos(2 * (Angle1 - Angle2));
            if (c > 0)  // sign(cos(...))
            {
                return 0; // <=> -1 event
            }
            return 1;// <=> +1 event

        }
        private int GetRandomAngleIndex()
        {
            return (int)(((float)NumberOfAnglesForStations) * h.GetRandom()); // pick an angleIndex
        }
        private double GetDelayTime(double c)
        {
            return Math.Ceiling(Math.Pow((1 - c * c), (d / 2)) * h.GetRandom() / tau); // delay time
        }
 
 
    }
    public enum Calculation
    {
        DeReat,
        Malus,
        Test,
    }
}
