﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Epr
{
    public class PolarizationFilter
    {
        public PolarizationFilter(int NumberOfAngles, int d, double tau)
        {
            //The polarization filter is instantieated having 'NumberOfAngles' possible number 
            //of random angles
            InitiateAngles(NumberOfAngles);
            this.d = d;
            this.tau = tau;
        }
        private double d;
        private double tau;
        private void InitiateAngles(int NumberOfAngles)
        {
            _Angles = new List<double>();
            for (int ii = 0; ii < NumberOfAngles; ii++)
            {
                _Angles.Add(h.GetRandomPiAngle());
            }
        }
        public void SetAnglesToZero()
        {
            for (int nn = 0; nn < this.Angles.Count;nn++ )
            {
                this.Angles[nn] = 0;
            }
        }
        /// <summary>
        /// Sets the internal index for the angles randomly to one of the possible values
        /// </summary>
        /// <returns>The random index</returns>
        public int ChooseRandomAngleIndex()
        {
            _currentAngleIndex = GetRandomAngleIndex();
            return _currentAngleIndex;
        }
        private int GetRandomAngleIndex()
        {
            return (int)(((float)_Angles.Count) * h.GetRandom()); // pick an angleIndex
        }
        /// <summary>
        /// When a particle hits the filter the chosen Interpretaion will be used for
        /// the actual handling in the filter
        /// </summary>
        /// <param name="Particle"></param>
        public void ParticleHit(Particle Particle)
        {
            switch (Calc_Epr.Interpretation)
            {
                case Calculation.DeReat:
                    ParticleHitfromDeRaet(Particle);
                    break;
                case Calculation.Malus:
                    ParticleHitMalus(Particle);
                    break;
                case Calculation.Test:
                    ParticleHitformTest(Particle);
                    break;
            }
        }
       private void ParticleHitfromDeRaet(Particle Particle)
        {
            double malus = h.Malus(Particle.Polarization - this.Angle);
            if (malus > 0)  // sign(cos(...))
            {
                Particle.Absorbed = true; // <=> -1 event
            }
            Particle.DelayTime = Math.Ceiling(Math.Pow((1 - malus * malus), (d / 2)) * h.GetRandom() / tau); // delay time
        }
        private void ParticleHitMalus(Particle Particle)
        {
            Particle.Absorbed = !h.MalusAbsorbed(Particle.Polarization - this.Angle);
        }
 
        private void ParticleHitformTest(Particle Particle)
        {
            //......

            Particle.Absorbed = true;
        }

        private int _currentAngleIndex = 0;

        private List<double> _Angles = new List<double>();
        public List<double> Angles
        {
            get
            {
                return _Angles;
            }
            set
            {
                _Angles = value;
            }
        }
        public double Angle
        {
            get
            {
                return _Angles[_currentAngleIndex];
            }
            set
            {
                _Angles[_currentAngleIndex] = value;
            }
        }

    }
}
